@@ -19,6 +19,11 @@
 #include "config.h"
 #include "mem_pool.h"
 #include "fstring.h"
+#include "util.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #define RSPAMD_EXPRESSION_MAX_PRIORITY 1024
 
@@ -26,17 +31,21 @@
 
 enum rspamd_expression_op {
 	OP_INVALID = 0,
-	OP_PLUS, /* || or + */
-	OP_MULT, /* && or * */
-	OP_OR, /* || or | */
-	OP_AND, /* && or & */
-	OP_NOT, /* ! */
-	OP_LT, /* < */
-	OP_GT, /* > */
-	OP_LE, /* <= */
-	OP_GE, /* >= */
+	OP_PLUS,   /* + */
+	OP_MULT,   /* * */
+	OP_MINUS,  /* - */
+	OP_DIVIDE, /* / */
+	OP_OR,     /* || or | */
+	OP_AND,    /* && or & */
+	OP_NOT,    /* ! */
+	OP_LT,     /* < */
+	OP_GT,     /* > */
+	OP_LE,     /* <= */
+	OP_GE,     /* >= */
+	OP_EQ,     /* == */
+	OP_NE,     /* != */
 	OP_OBRACE, /* ( */
-	OP_CBRACE /* ) */
+	OP_CBRACE  /* ) */
 };
 
 typedef struct rspamd_expression_atom_s {
@@ -45,26 +54,30 @@ typedef struct rspamd_expression_atom_s {
 	/* Opaque userdata */
 	gpointer data;
 	/* String representation of atom */
-	const gchar *str;
+	const char *str;
 	/* Length of the string representation of atom */
-	gsize len;
-	/* Average execution time (in ticks) */
-	gdouble avg_ticks;
-	/* Amount of positive triggers */
-	guint hits;
+	unsigned int len;
 	/* Relative priority */
-	gint priority;
+	int priority;
+	unsigned int hits;
+	struct rspamd_counter_data exec_time;
 } rspamd_expression_atom_t;
 
+typedef double (*rspamd_expression_process_cb)(gpointer runtime_data,
+											   rspamd_expression_atom_t *atom);
+
 struct rspamd_atom_subr {
 	/* Parses atom from string and returns atom structure */
-	rspamd_expression_atom_t * (*parse)(const gchar *line, gsize len,
-			rspamd_mempool_t *pool, gpointer ud, GError **err);
+	rspamd_expression_atom_t *(*parse)(const char *line, gsize len,
+									   rspamd_mempool_t *pool, gpointer ud, GError **err);
+
 	/* Process atom via the opaque pointer (e.g. struct rspamd_task *) */
-	gdouble (*process) (gpointer input, rspamd_expression_atom_t *atom);
+	rspamd_expression_process_cb process;
+
 	/* Calculates the relative priority of the expression */
-	gint (*priority) (rspamd_expression_atom_t *atom);
-	void (*destroy) (rspamd_expression_atom_t *atom);
+	int (*priority)(rspamd_expression_atom_t *atom);
+
+	void (*destroy)(rspamd_expression_atom_t *atom);
 };
 
 /* Opaque structure */
@@ -81,10 +94,10 @@ struct rspamd_expression;
  * @param target the target expression
  * @return TRUE if an expression have been parsed
  */
-gboolean rspamd_parse_expression (const gchar *line, gsize len,
-		const struct rspamd_atom_subr *subr, gpointer subr_data,
-		rspamd_mempool_t *pool, GError **err,
-		struct rspamd_expression **target);
+gboolean rspamd_parse_expression(const char *line, gsize len,
+								 const struct rspamd_atom_subr *subr, gpointer subr_data,
+								 rspamd_mempool_t *pool, GError **err,
+								 struct rspamd_expression **target);
 
 /**
  * Process the expression and return its value using atom 'process' functions with the specified data pointer
@@ -92,8 +105,9 @@ gboolean rspamd_parse_expression (const gchar *line, gsize len,
  * @param data opaque data pointer for all the atoms
  * @return the value of expression
  */
-gdouble rspamd_process_expression (struct rspamd_expression *expr, gint flags,
-		gpointer data);
+double rspamd_process_expression(struct rspamd_expression *expr,
+								 int flags,
+								 gpointer runtime_ud);
 
 /**
  * Process the expression and return its value using atom 'process' functions with the specified data pointer.
@@ -103,22 +117,37 @@ gdouble rspamd_process_expression (struct rspamd_expression *expr, gint flags,
  * @param track pointer array to atoms tracking
  * @return the value of expression
  */
-gdouble rspamd_process_expression_track (struct rspamd_expression *expr, gint flags,
-		gpointer data, GPtrArray *track);
+double rspamd_process_expression_track(struct rspamd_expression *expr,
+									   int flags,
+									   gpointer runtime_ud,
+									   GPtrArray **track);
+
+/**
+ * Process the expression with the custom processor
+ * @param expr
+ * @param cb
+ * @param process_data
+ * @return
+ */
+double rspamd_process_expression_closure(struct rspamd_expression *expr,
+										 rspamd_expression_process_cb cb,
+										 int flags,
+										 gpointer runtime_ud,
+										 GPtrArray **track);
 
 /**
  * Shows string representation of an expression
  * @param expr expression to show
  * @return freshly allocated string with expression
  */
-GString *rspamd_expression_tostring (struct rspamd_expression *expr);
+GString *rspamd_expression_tostring(struct rspamd_expression *expr);
 
 /**
  * Callback that is called on @see rspamd_expression_atom_foreach, atom is ephemeral
  * and should not be modified within callback
  */
-typedef void (*rspamd_expression_atom_foreach_cb) (const rspamd_ftok_t *atom,
-		gpointer ud);
+typedef void (*rspamd_expression_atom_foreach_cb)(const rspamd_ftok_t *atom,
+												  gpointer ud);
 
 /**
  * Traverse over all atoms in the expression
@@ -126,8 +155,24 @@ typedef void (*rspamd_expression_atom_foreach_cb) (const rspamd_ftok_t *atom,
  * @param cb callback to be called
  * @param ud opaque data passed to `cb`
  */
-void rspamd_expression_atom_foreach (struct rspamd_expression *expr,
-		rspamd_expression_atom_foreach_cb cb, gpointer cbdata);
+void rspamd_expression_atom_foreach(struct rspamd_expression *expr,
+									rspamd_expression_atom_foreach_cb cb, gpointer cbdata);
+
+/**
+ * Extended callback that provides access to the AST node (for checking parent operations like NOT)
+ */
+typedef void (*rspamd_expression_atom_foreach_cb_ex)(GNode *atom_node,
+													 rspamd_expression_atom_t *atom,
+													 gpointer ud);
+
+/**
+ * Traverse over all atoms in the expression with access to AST nodes
+ * @param expr expression
+ * @param cb callback to be called with GNode and full atom structure
+ * @param ud opaque data passed to `cb`
+ */
+void rspamd_expression_atom_foreach_ex(struct rspamd_expression *expr,
+									   rspamd_expression_atom_foreach_cb_ex cb, gpointer cbdata);
 
 /**
  * Checks if a specified node in AST is the specified operation
@@ -135,6 +180,10 @@ void rspamd_expression_atom_foreach (struct rspamd_expression *expr,
  * @param op operation to check
  * @return TRUE if node is operation node and is exactly the specified option
  */
-gboolean rspamd_expression_node_is_op (GNode *node, enum rspamd_expression_op op);
+gboolean rspamd_expression_node_is_op(GNode *node, enum rspamd_expression_op op);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* SRC_LIBUTIL_EXPRESSION_H_ */
